@@ -18,6 +18,7 @@ import coinstac_dinunet.utils.tensorutils as _tu
 import coinstac_dinunet.vision.plotter as _plot
 from coinstac_dinunet.config.status import *
 from coinstac_dinunet.utils.logger import *
+from coinstac_dinunet.utils.utils import stop_training_
 
 
 class NNTrainer:
@@ -120,11 +121,11 @@ class NNTrainer:
             self.nn[k].eval()
 
         _cache = {**self.cache}
-        _cache['shuffle'] = False
+        _cache.update(shuffle=False)
         _cache['mode'] = mode
         eval_avg, eval_metrics = self.new_averages(), self.new_metrics()
         eval_loaders = []
-        for dataset in dataset_list:
+        for dataset in [d for d in dataset_list if d is not None]:
             _cache['batch_size'] = _tu.get_safe_batch_size(self.cache['batch_size'], len(dataset))
             eval_loaders.append(_data.COINNDataLoader.new(dataset=dataset, **_cache))
 
@@ -134,15 +135,9 @@ class NNTrainer:
                 metrics = self.new_metrics()
                 avg = self.new_averages()
                 for i, batch in enumerate(loader, 1):
-
                     it = self.iteration(batch)
-                    if not it.get('metrics'):
-                        it['metrics'] = _base_metrics.COINNMetrics()
-
-                    if not it.get('averages'):
-                        it['averages'] = _base_metrics.COINNAverages()
-
                     metrics.accumulate(it['metrics']), avg.accumulate(it['averages'])
+
                     if save_pred:
                         its.append(it)
 
@@ -181,12 +176,12 @@ class NNTrainer:
         cache[Key.TRAIN_LOG] = []
         cache[Key.VALIDATION_LOG] = []
         metric_direction = self.cache['monitor_metric'][1]
-        self.cache['best_local_epoch'] = 0
-        self.cache.update(best_local_score=0.0 if metric_direction == 'maximize' else _conf.max_size)
+        self.cache['best_val_epoch'] = 0
+        self.cache.update(best_val_score=0.0 if metric_direction == 'maximize' else _conf.max_size)
 
         _dset_cache = {**self.cache}
-        _dset_cache['mode'] = 'train'
-        _dset_cache['shuffle'] = True
+        _dset_cache.update(mode=Mode.TRAIN)
+        _dset_cache.update(shuffle=True)
         dataset = self._get_train_dataset(dataset_cls)
         _dset_cache['batch_size'] = _tu.get_safe_batch_size(self.cache['batch_size'], len(dataset))
         loader = _data.COINNDataLoader.new(dataset=dataset, **_dset_cache)
@@ -205,10 +200,6 @@ class NNTrainer:
                 its.append(self.training_iteration_local(i, batch))
                 if i % local_iter == 0:
                     it = self._reduce_iteration(its)
-                    if not it.get('metrics'):
-                        it['metrics'] = _base_metrics.COINNMetrics()
-                    if not it.get('averages'):
-                        it['averages'] = _base_metrics.COINNAverages()
 
                     ep_avg.accumulate(it['averages']), ep_metrics.accumulate(it['metrics'])
                     _avg.accumulate(it['averages']), _metrics.accumulate(it['metrics'])
@@ -221,24 +212,26 @@ class NNTrainer:
                         _metrics.reset(), _avg.reset()
                     self._on_iteration_end(i=_i, ep=ep, it=it)
 
-            val_averages, val_metric = self.evaluation(mode='validation',
-                                                       dataset_list=[self._get_validation_dataset(dataset_cls)])
-            cache[Key.VALIDATION_LOG].append([*val_averages.get(), *val_metric.get()])
-            out.update(**self._save_if_better(ep, val_metric))
+            if ep % self.cache.get('validation_epochs', 1) == 0:
+                val_averages, val_metric = self.evaluation(mode='validation',
+                                                           dataset_list=[self._get_validation_dataset(dataset_cls)])
+                cache[Key.VALIDATION_LOG].append([*val_averages.get(), *val_metric.get()])
+                out.update(**self._save_if_better(ep, val_metric))
 
-            self._on_epoch_end(ep=ep, ep_averages=ep_avg, ep_metrics=ep_metrics,
-                               val_averages=val_averages, val_metrics=val_metric)
+                self._on_epoch_end(ep=ep, ep_averages=ep_avg, ep_metrics=ep_metrics,
+                                   val_averages=val_averages, val_metrics=val_metric)
 
-            if lazy_debug(ep):
-                self._save_progress(cache, epoch=ep)
+                if lazy_debug(ep):
+                    self._save_progress(cache, epoch=ep)
 
-            if self._stop_early(epoch=ep, epoch_averages=ep_avg, epoch_metrics=ep_metrics,
-                                validation_averages=val_averages, validation_metric=val_metric):
-                break
+                if self._stop_early(ep, val_metric, val_averages=val_averages,
+                                    epoch_averages=ep_avg, epoch_metrics=ep_metrics):
+                    break
 
         self._save_progress(cache, epoch=ep)
-        cache['best_local_epoch'] = self.cache['best_local_epoch']
-        cache['best_local_score'] = self.cache['best_local_score']
+        cache['best_val_epoch'] = self.cache['best_val_epoch']
+        cache['best_val_score'] = self.cache['best_val_score']
+        cache.update(**self.cache)
         _utils.save_cache(cache, self.cache['log_dir'])
         return out
 
@@ -297,7 +290,7 @@ class NNTrainer:
         return reduced
 
     def _set_monitor_metric(self):
-        self.cache['monitor_metric'] = 'f1', 'maximize'
+        self.cache['monitor_metric'] = 'time', 'maximize'
 
     def _load_dataset(self, dataset_cls, split_key):
         dataset = dataset_cls(mode=split_key, limit=self.cache.get('load_limit', _conf.max_size))
@@ -321,16 +314,16 @@ class NNTrainer:
     def _get_test_dataset(self, dataset_cls):
         return self._load_dataset(dataset_cls, split_key='test')
 
-    def _save_if_better(self, epoch, metrics):
+    def _save_if_better(self, epoch, val_metrics):
         return {}
 
     def new_metrics(self):
-        return _base_metrics.Prf1a()
+        return _base_metrics.COINNMetrics()
 
     def new_averages(self):
         return _base_metrics.COINNAverages(num_averages=1)
 
-    def _on_epoch_end(self, ep, ep_averages, ep_metrics, val_averages, val_metrics):
+    def _on_epoch_end(self, ep, **kw):
         r"""
         Any logic to run after an epoch ends.
         """
@@ -343,15 +336,12 @@ class NNTrainer:
         return {}
 
     def _save_progress(self, cache, epoch):
-        _plot.plot_progress(cache, self.cache['log_dir'], plot_keys=[Key.TRAIN_LOG, Key.VALIDATION_LOG], epoch=epoch)
+        _plot.plot_progress(cache, self.cache['log_dir'], plot_keys=[Key.TRAIN_LOG], epoch=epoch)
+        _plot.plot_progress(cache, self.cache['log_dir'], plot_keys=[Key.VALIDATION_LOG],
+                            epoch=epoch // self.cache['validation_epochs'])
 
-    def _stop_early(self, **kw):
-        r"""
-        Stop the training based on some criteria.
-         For example: the implementation below will stop training if the validation
-         scores does not improve within a 'patience' number of epochs.
-        """
-        return kw.get('epoch') - self.cache['best_local_epoch'] >= self.cache['args'].get('patience', 'epochs')
+    def _stop_early(self, epoch, val_metrics=None, **kw):
+        return stop_training_(epoch, self.cache)
 
     def _set_log_headers(self):
-        self.cache['log_header'] = 'Loss,Accuracy,F1,Precision,Recall'
+        self.cache['log_header'] = 'Loss,Accuracy'
